#! /bin/bash

CHECK_SERVICES=true
RESTART_SERVICES=false
CHECK_ETC=true

export ABORT_CODE=99
export LANG=C
set -u

update_fehler_liste=""
upgrade_fehler_liste=""
interrupted=false
args=""
[ $# -gt 0 ] && args=$1


if $CHECK_SERVICES ; then
	services_before=`mktemp -t vserver-dist-upgrade.services_before.XXXX`
	services_after=`mktemp -t vserver-dist-upgrade.services_after.XXXX`
	services_diff=`mktemp -t vserver-dist-upgrade.services_diff.XXXX`
fi

endausgabe_func () {
        echo
        [ -n "$update_fehler_liste" ] && echo "Update-Fehler:$update_fehler_liste"
        [ -n "$upgrade_fehler_liste" ] && echo "Upgrade-Fehler:$upgrade_fehler_liste"
	if $CHECK_SERVICES ; then
		rm -f $services_before
		rm -f $services_after
		rm -f $services_diff
	fi
}

ask_cont () {
	echo
        while read -p 'weiter?(Y|n) ' ; do
		REPLY=${REPLY:0:1}
                case "$REPLY" in
                        y|Y|j|J|'') 
                                return
                        ;;
                        n|N)
                                break
                        ;;
                esac
        done
        endausgabe_func
        exit
}

int_func () {
	echo "Interrupted by ctrl-C"
	interrupted=true
	ask_cont
}


search_running_services_func () {
	# try my best ..
	vps  cft '?' | grep -v '\_' | awk '{print $2,$NF}' | grep ^$context | awk '{print $NF}' | sort -u
}

find_missing_services_func () {
	sleep 2
	search_running_services_func >$services_after
	diff $services_before $services_after >$services_diff
	missing_servs=`grep '^<' $services_diff | cut -d' ' -f2`
}

trap int_func SIGINT


for v in `cat /etc/vservers/*/name`; do 
	vserver $v running && {
		context=`cat /etc/vservers/$v/context`
		echo
		echo $v - update
		echo '--------' 
		interrupted=false
		vserver $v exec aptitude update
                status=$?
                if [ $status -ne 0 ] ; then
                        update_fehler_liste+="  $v"
			if ! $interrupted; then
                        	echo "$v: aptitude update endet mit Fehler-Status $status"
                        	ask_cont
			fi
                fi
		interrupted=false
		$CHECK_ETC && vserver $v exec /bin/bash -c "type check-etc-rcs-files" && vserver $v exec check-etc-rcs-files --before-dist-upgrade
		[ $? -eq $ABORT_CODE ] && ask_cont
		echo $v - upgrade
		echo '========' 
		$CHECK_SERVICES && search_running_services_func >$services_before
		vserver $v exec aptitude -V $args dist-upgrade
                status=$?
                if [ $status -ne 0 ] ; then
                        upgrade_fehler_liste+="  $v"
			if ! $interrupted; then
                        	echo "$v: aptitude upgrade endet mit Fehler-Status $status"
                        	ask_cont
			fi
                fi
		interrupted=false
		$CHECK_ETC && vserver $v exec /bin/bash -c "type check-etc-rcs-files" && vserver $v exec check-etc-rcs-files --after-dist-upgrade
		[ $? -eq $ABORT_CODE ] && ask_cont
		vserver $v exec aptitude clean
		if $CHECK_SERVICES ; then 
			find_missing_services_func
			if [ -n "$missing_servs" ] ; then
				if $RESTART_SERVICES; then
					for s in $missing_servs; do
						[ "$s" = master ] && s=postfix
						if [ -x /etc/init.d/$s ] ; then
							/etc/init.d/$s start 
						elif s=${s%-*}; [ -x /etc/init.d/$s ] ; then
							/etc/init.d/$s start
						fi
					done
				find_missing_services_func
				fi
				if [ -n "$missing_servs" ] ; then
					echo
					echo "services not running after upgrade: "$missing_servs
					ask_cont
				fi
			fi
			new_servs=`grep '^>' $services_diff | cut -d' ' -f2` 
			if [ -n "$new_servs" ] ; then
				echo
				echo "new services running after upgrade: "$new_servs
				ask_cont
			fi
		fi
	} 
done

endausgabe_func

